@@ -34,6 +34,11 @@
 #include <unistd.h>
 
 #include <gdk/gdk.h>
+#include <gtk/gtk.h>
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+#include <cairo/cairo-xlib.h>
+#endif
 
 #include "display.h"
 #include "screen.h"
@@ -162,6 +167,8 @@ typedef struct _MetaCompWindow
   gboolean damaged;
   gboolean shaped;
 
+  XRectangle shape_bounds;
+
   MetaCompWindowType type;
 
   Damage damage;
@@ -1796,6 +1803,11 @@ add_win (MetaScreen *screen,
   cw->damaged = FALSE;
   cw->shaped = is_shaped (display, xwindow);
 
+  cw->shape_bounds.x = cw->attrs.x;
+  cw->shape_bounds.y = cw->attrs.y;
+  cw->shape_bounds.width = cw->attrs.width;
+  cw->shape_bounds.height = cw->attrs.height;
+
   if (cw->attrs.class == InputOnly)
     cw->damage = None;
   else
@@ -1940,6 +1952,7 @@ resize_win (MetaCompWindow *cw,
   Display *xdisplay = meta_display_get_xdisplay (display);
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
   XserverRegion damage;
+  XserverRegion shape;
   gboolean debug;
 
   debug = DISPLAY_COMPOSITOR (display)->debug;
@@ -2031,6 +2044,10 @@ resize_win (MetaCompWindow *cw,
       XFixesCopyRegion (xdisplay, damage, cw->extents);
     }
 
+  shape = XFixesCreateRegion (xdisplay, &cw->shape_bounds, 1);
+  XFixesUnionRegion (xdisplay, damage, damage, shape);
+  XFixesDestroyRegion (xdisplay, shape);
+
   dump_xserver_region ("resize_win", display, damage);
   add_damage (screen, damage);
 
@@ -2389,6 +2406,21 @@ process_shape (MetaCompositorXRender *compositor,
 
       if (event->shaped && !cw->shaped)
         cw->shaped = TRUE;
+
+      if (event->shaped == True)
+        {
+          cw->shape_bounds.x = cw->attrs.x + event->x;
+          cw->shape_bounds.y = cw->attrs.y + event->y;
+          cw->shape_bounds.width = event->width;
+          cw->shape_bounds.height = event->height;
+        }
+      else
+        {
+          cw->shape_bounds.x = cw->attrs.x;
+          cw->shape_bounds.y = cw->attrs.y;
+          cw->shape_bounds.width = cw->attrs.width;
+          cw->shape_bounds.height = cw->attrs.height;
+        }
     }
 }
 
@@ -2770,6 +2802,46 @@ xrender_process_event (MetaCompositor *compositor,
 #endif
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static cairo_surface_t *
+xrender_get_window_surface (MetaCompositor *compositor,
+                            MetaWindow     *window)
+{
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  MetaFrame *frame;
+  Window xwindow;
+  MetaScreen *screen;
+  MetaCompWindow *cw;
+  MetaCompositorXRender *xrc;
+  Display *display;
+  Pixmap pixmap;
+
+  frame = meta_window_get_frame (window);
+
+  if (frame)
+    xwindow = meta_frame_get_xwindow (frame);
+  else
+    xwindow = meta_window_get_xwindow (window);
+
+  screen = meta_window_get_screen (window);
+  cw = find_window_for_screen (screen, xwindow);
+
+  if (cw == NULL)
+    return NULL;
+
+  xrc = (MetaCompositorXRender *) compositor;
+  display = meta_display_get_xdisplay (xrc->display);
+
+  if (meta_window_is_shaded (window))
+    pixmap = cw->shaded_back_pixmap;
+  else
+    pixmap = cw->back_pixmap;
+
+  return cairo_xlib_surface_create (display, pixmap, cw->attrs.visual,
+                                    cw->attrs.width, cw->attrs.height);
+#endif
+}
+#else
 static Pixmap
 xrender_get_window_pixmap (MetaCompositor *compositor,
                            MetaWindow     *window)
@@ -2790,6 +2862,7 @@ xrender_get_window_pixmap (MetaCompositor *compositor,
     return cw->back_pixmap;
 #endif
 }
+#endif  /* GTK_CHECK_VERSION */
 
 static void
 xrender_set_active_window (MetaCompositor *compositor,
@@ -2973,7 +3046,11 @@ static MetaCompositor comp_info = {
   xrender_remove_window,
   xrender_set_updates,
   xrender_process_event,
+#if GTK_CHECK_VERSION (3, 0, 0)
+  xrender_get_window_surface,
+#else
   xrender_get_window_pixmap,
+#endif
   xrender_set_active_window,
   xrender_free_window,
   xrender_maximize_window,
