@@ -2,9 +2,9 @@
 
 /* Marco theme preview widget */
 
-/* 
+/*
  * Copyright (C) 2002 Havoc Pennington
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -14,7 +14,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
@@ -37,10 +37,10 @@ static void meta_preview_get_preferred_width(GtkWidget *widget,
                                              gint *minimal, gint *natural);
 static void meta_preview_get_preferred_height(GtkWidget *widget,
                                               gint *minimal, gint *natural);
-#endif
-
+#else
 static void     meta_preview_size_request  (GtkWidget        *widget,
                                             GtkRequisition   *req);
+#endif
 static void     meta_preview_size_allocate (GtkWidget        *widget,
                                             GtkAllocation    *allocation);
 #if GTK_CHECK_VERSION(3, 0, 0)
@@ -52,6 +52,8 @@ static gboolean meta_preview_expose        (GtkWidget        *widget,
 #endif
 static void     meta_preview_finalize      (GObject          *object);
 
+#define NO_CHILD_WIDTH 80
+#define NO_CHILD_HEIGHT 20
 
 #if GTK_CHECK_VERSION(3, 0, 0)
 
@@ -101,14 +103,16 @@ meta_preview_class_init (MetaPreviewClass *class)
 
   gobject_class->finalize = meta_preview_finalize;
 
-  #if GTK_CHECK_VERSION(3, 0, 0)
+#if GTK_CHECK_VERSION(3, 0, 0)
   widget_class->draw = meta_preview_draw;
   widget_class->get_preferred_width = meta_preview_get_preferred_width;
   widget_class->get_preferred_height = meta_preview_get_preferred_height;
-  #else
+
+  gtk_container_class_handle_border_width (GTK_CONTAINER_CLASS (class));
+#else
   widget_class->expose_event = meta_preview_expose;
   widget_class->size_request = meta_preview_size_request;
-  #endif
+#endif
   widget_class->size_allocate = meta_preview_size_allocate;
 }
 
@@ -116,7 +120,7 @@ static void
 meta_preview_init (MetaPreview *preview)
 {
   int i;
-  
+
   gtk_widget_set_has_window (GTK_WIDGET (preview), FALSE);
 
   i = 0;
@@ -126,13 +130,13 @@ meta_preview_init (MetaPreview *preview)
       preview->button_layout.right_buttons[i] = META_BUTTON_FUNCTION_LAST;
       ++i;
     }
-  
+
   preview->button_layout.left_buttons[0] = META_BUTTON_FUNCTION_MENU;
 
   preview->button_layout.right_buttons[0] = META_BUTTON_FUNCTION_MINIMIZE;
   preview->button_layout.right_buttons[1] = META_BUTTON_FUNCTION_MAXIMIZE;
   preview->button_layout.right_buttons[2] = META_BUTTON_FUNCTION_CLOSE;
-  
+
   preview->type = META_FRAME_TYPE_NORMAL;
   preview->flags =
     META_FRAME_ALLOWS_DELETE |
@@ -144,7 +148,7 @@ meta_preview_init (MetaPreview *preview)
     META_FRAME_HAS_FOCUS |
     META_FRAME_ALLOWS_SHADE |
     META_FRAME_ALLOWS_MOVE;
-  
+
   preview->left_width = -1;
   preview->right_width = -1;
   preview->top_height = -1;
@@ -155,13 +159,9 @@ GtkWidget*
 meta_preview_new (void)
 {
   MetaPreview *preview;
-  
-  #if GTK_CHECK_VERSION(3, 0, 0)
+
   preview = g_object_new (META_TYPE_PREVIEW, NULL);
-  #else
-  preview = gtk_type_new (META_TYPE_PREVIEW);
-  #endif
-  
+
   return GTK_WIDGET (preview);
 }
 
@@ -174,7 +174,7 @@ meta_preview_finalize (GObject *object)
 
   g_free (preview->title);
   preview->title = NULL;
-  
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -184,7 +184,7 @@ ensure_info (MetaPreview *preview)
   GtkWidget *widget;
 
   widget = GTK_WIDGET (preview);
-  
+
   if (preview->layout == NULL)
     {
       PangoFontDescription *font_desc;
@@ -192,34 +192,34 @@ ensure_info (MetaPreview *preview)
       PangoAttrList *attrs;
       PangoAttribute *attr;
 
-      if (preview->theme)        
+      if (preview->theme)
         scale = meta_theme_get_title_scale (preview->theme,
                                             preview->type,
                                             preview->flags);
       else
         scale = 1.0;
-      
+
       preview->layout = gtk_widget_create_pango_layout (widget,
                                                         preview->title);
-      
+
       font_desc = meta_gtk_widget_get_font_desc (widget, scale, NULL);
-      
+
       preview->text_height =
         meta_pango_font_desc_get_text_height (font_desc,
                                               gtk_widget_get_pango_context (widget));
-          
+
       attrs = pango_attr_list_new ();
-      
+
       attr = pango_attr_size_new (pango_font_description_get_size (font_desc));
       attr->start_index = 0;
       attr->end_index = G_MAXINT;
-      
+
       pango_attr_list_insert (attrs, attr);
-      
+
       pango_layout_set_attributes (preview->layout, attrs);
-      
-      pango_attr_list_unref (attrs);      
-  
+
+      pango_attr_list_unref (attrs);
+
       pango_font_description_free (font_desc);
     }
 
@@ -247,53 +247,46 @@ ensure_info (MetaPreview *preview)
 }
 #if GTK_CHECK_VERSION(3, 0, 0)
 
-static void meta_preview_get_preferred_width(GtkWidget *widget, gint *minimal, gint *natural)
-{
-    GtkRequisition requisition;
-    meta_preview_size_request (widget, &requisition);
-    *minimal = *natural = requisition.width;
-}
-
-static void meta_preview_get_preferred_height(GtkWidget *widget, gint *minimal, gint *natural)
-{
-    GtkRequisition requisition;
-    meta_preview_size_request (widget, &requisition);
-    *minimal = *natural = requisition.height;
-}
-
-
 static gboolean
-meta_preview_draw(GtkWidget   *widget,
-				  cairo_t     *cr)
+meta_preview_draw (GtkWidget *widget,
+                   cairo_t   *cr)
 {
- MetaPreview *preview = META_PREVIEW (widget);
+  MetaPreview *preview;
   GtkAllocation allocation;
+  int border_width;
+  int client_width;
+  int client_height;
+  MetaButtonState button_states[META_BUTTON_TYPE_LAST] =
+  {
+    META_BUTTON_STATE_NORMAL,
+    META_BUTTON_STATE_NORMAL,
+    META_BUTTON_STATE_NORMAL,
+    META_BUTTON_STATE_NORMAL
+  };
+
+  g_return_val_if_fail (META_IS_PREVIEW (widget), FALSE);
+
+  preview = META_PREVIEW (widget);
+
+  ensure_info (preview);
+
+  cairo_save (cr);
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
   gtk_widget_get_allocation (widget, &allocation);
+  client_width = allocation.width - preview->left_width - preview->right_width - border_width * 2;
+  client_height = allocation.height - preview->top_height - preview->bottom_height - border_width * 2;
+
+  if (client_width < 0)
+    client_width = 1;
+  if (client_height < 0)
+    client_height = 1;
 
   if (preview->theme)
     {
-      int client_width;
-      int client_height;
-      MetaButtonState button_states[META_BUTTON_TYPE_LAST] =
-      {
-        META_BUTTON_STATE_NORMAL,
-        META_BUTTON_STATE_NORMAL,
-        META_BUTTON_STATE_NORMAL,
-        META_BUTTON_STATE_NORMAL
-      };
-  
-      ensure_info (preview);
-      cairo_save (cr);
+      border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
-      client_width = allocation.width - preview->left_width - preview->right_width;
-      client_height = allocation.height - preview->top_height - preview->bottom_height;
-
-      if (client_width < 0)
-        client_width = 1;
-      if (client_height < 0)
-        client_height = 1;  
-      
       meta_theme_draw_frame (preview->theme,
                              widget,
                              cr,
@@ -306,11 +299,84 @@ meta_preview_draw(GtkWidget   *widget,
                              button_states,
                              meta_preview_get_mini_icon (),
                              meta_preview_get_icon ());
-
-      cairo_restore (cr);
     }
 
-  return GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
+  cairo_restore (cr);
+
+  /* draw child */
+  return GTK_WIDGET_CLASS (meta_preview_parent_class)->draw (widget, cr);
+}
+
+static void
+meta_preview_get_preferred_width (GtkWidget *widget,
+                                  gint      *minimum,
+                                  gint      *natural)
+{
+  MetaPreview *preview;
+  int border_width;
+  GtkWidget *child;
+
+  preview = META_PREVIEW (widget);
+
+  ensure_info (preview);
+
+  *minimum = *natural = preview->left_width + preview->right_width;
+
+  child = gtk_bin_get_child (GTK_BIN (preview));
+  if (child && gtk_widget_get_visible (child))
+    {
+      gint child_min, child_nat;
+
+      gtk_widget_get_preferred_width (child, &child_min, &child_nat);
+
+      *minimum += child_min;
+      *natural += child_nat;
+    }
+  else
+    {
+      *minimum += NO_CHILD_WIDTH;
+      *natural += NO_CHILD_WIDTH;
+    }
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  *minimum += border_width * 2;
+  *natural += border_width * 2;
+}
+
+static void
+meta_preview_get_preferred_height (GtkWidget *widget,
+                                   gint      *minimum,
+                                   gint      *natural)
+{
+  MetaPreview *preview;
+  int border_width;
+  GtkWidget *child;
+
+  preview = META_PREVIEW (widget);
+
+  ensure_info (preview);
+
+  *minimum = *natural = preview->top_height + preview->bottom_height;
+
+  child = gtk_bin_get_child (GTK_BIN (preview));
+  if (child && gtk_widget_get_visible (child))
+    {
+      gint child_min, child_nat;
+
+      gtk_widget_get_preferred_height (child, &child_min, &child_nat);
+
+      *minimum += child_min;
+      *natural += child_nat;
+    }
+  else
+    {
+      *minimum += NO_CHILD_HEIGHT;
+      *natural += NO_CHILD_HEIGHT;
+    }
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  *minimum += border_width * 2;
+  *natural += border_width * 2;
 }
 
 #else
@@ -331,7 +397,7 @@ meta_preview_expose (GtkWidget      *widget,
     META_BUTTON_STATE_NORMAL,
     META_BUTTON_STATE_NORMAL
   };
-  
+
   g_return_val_if_fail (META_IS_PREVIEW (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -340,7 +406,7 @@ meta_preview_expose (GtkWidget      *widget,
   ensure_info (preview);
 
   border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  
+
   gtk_widget_get_allocation (widget, &allocation);
   client_width = allocation.width - preview->left_width - preview->right_width - border_width * 2;
   client_height = allocation.height - preview->top_height - preview->bottom_height - border_width * 2;
@@ -348,12 +414,12 @@ meta_preview_expose (GtkWidget      *widget,
   if (client_width < 0)
     client_width = 1;
   if (client_height < 0)
-    client_height = 1;  
-  
+    client_height = 1;
+
   if (preview->theme)
     {
       border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-      
+
       meta_theme_draw_frame (preview->theme,
                              widget,
                              gtk_widget_get_window (widget),
@@ -375,8 +441,6 @@ meta_preview_expose (GtkWidget      *widget,
   return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
 }
 
-#endif
-
 static void
 meta_preview_size_request (GtkWidget      *widget,
                            GtkRequisition *req)
@@ -391,7 +455,7 @@ meta_preview_size_request (GtkWidget      *widget,
 
   req->width = preview->left_width + preview->right_width;
   req->height = preview->top_height + preview->bottom_height;
-  
+
   child = gtk_bin_get_child (GTK_BIN (preview));
   if (child &&
       gtk_widget_get_visible (child))
@@ -405,8 +469,6 @@ meta_preview_size_request (GtkWidget      *widget,
     }
   else
     {
-#define NO_CHILD_WIDTH 80
-#define NO_CHILD_HEIGHT 20
       req->width += NO_CHILD_WIDTH;
       req->height += NO_CHILD_HEIGHT;
     }
@@ -415,6 +477,7 @@ meta_preview_size_request (GtkWidget      *widget,
   req->width += border_width * 2;
   req->height += border_width * 2;
 }
+#endif
 
 static void
 meta_preview_size_allocate (GtkWidget         *widget,
@@ -422,29 +485,27 @@ meta_preview_size_allocate (GtkWidget         *widget,
 {
   MetaPreview *preview;
   int border_width;
-  GtkAllocation child_allocation;
+  GtkAllocation widget_allocation, child_allocation;
   GtkWidget *child;
-  
+
   preview = META_PREVIEW (widget);
 
   ensure_info (preview);
-  
-  #if !GTK_CHECK_VERSION(3, 0, 0)
-  widget->allocation = *allocation;
-  #endif
+
+  gtk_widget_set_allocation (widget, allocation);
 
   border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  
+
   child = gtk_bin_get_child (GTK_BIN (widget));
   if (child &&
       gtk_widget_get_visible (child))
     {
-      child_allocation.x = allocation->x + border_width + preview->left_width;
-      child_allocation.y = allocation->y + border_width + preview->top_height;
-      
-      child_allocation.width = MAX (1, allocation->width - border_width * 2 - preview->left_width - preview->right_width);
-      child_allocation.height = MAX (1, allocation->height - border_width * 2 - preview->top_height - preview->bottom_height);
+      gtk_widget_get_allocation (widget, &widget_allocation);
+      child_allocation.x = widget_allocation.x + border_width + preview->left_width;
+      child_allocation.y = widget_allocation.y + border_width + preview->top_height;
 
+      child_allocation.width = MAX (1, widget_allocation.width - border_width * 2 - preview->left_width - preview->right_width);
+      child_allocation.height = MAX (1, widget_allocation.height - border_width * 2 - preview->top_height - preview->bottom_height);
       gtk_widget_size_allocate (gtk_bin_get_child (GTK_BIN (widget)), &child_allocation);
     }
 }
@@ -471,7 +532,7 @@ meta_preview_set_theme (MetaPreview    *preview,
   g_return_if_fail (META_IS_PREVIEW (preview));
 
   preview->theme = theme;
-  
+
   clear_cache (preview);
 
   gtk_widget_queue_resize (GTK_WIDGET (preview));
@@ -485,7 +546,7 @@ meta_preview_set_title (MetaPreview    *preview,
 
   g_free (preview->title);
   preview->title = g_strdup (title);
-  
+
   clear_cache (preview);
 
   gtk_widget_queue_resize (GTK_WIDGET (preview));
@@ -522,9 +583,9 @@ meta_preview_set_button_layout (MetaPreview            *preview,
                                 const MetaButtonLayout *button_layout)
 {
   g_return_if_fail (META_IS_PREVIEW (preview));
-  
-  preview->button_layout = *button_layout;  
-  
+
+  preview->button_layout = *button_layout;
+
   gtk_widget_queue_draw (GTK_WIDGET (preview));
 }
 
@@ -550,14 +611,14 @@ meta_preview_get_icon (void)
                                                    NULL);
       else
           default_icon = gtk_icon_theme_load_icon (theme,
-                                                   "gtk-missing-image",
+                                                   "image-missing",
                                                    META_ICON_WIDTH,
                                                    0,
                                                    NULL);
 
       g_assert (default_icon);
     }
-  
+
   return default_icon;
 }
 
@@ -583,14 +644,14 @@ meta_preview_get_mini_icon (void)
                                                    NULL);
       else
           default_icon = gtk_icon_theme_load_icon (theme,
-                                                   "gtk-missing-image",
+                                                   "image-missing",
                                                    META_MINI_ICON_WIDTH,
                                                    0,
                                                    NULL);
 
       g_assert (default_icon);
     }
-  
+
   return default_icon;
 }
 
@@ -704,4 +765,4 @@ meta_preview_get_clip_region (MetaPreview *preview, gint new_window_width, gint
   gdk_region_destroy (corners_xregion);
 
   return window_xregion;
-}
\ No newline at end of file
+}
