@@ -258,7 +258,9 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   GtkWidget *table;
 #endif
   GtkWidget *vbox;
+#if !GTK_CHECK_VERSION (3, 14, 0)
   GtkWidget *align;
+#endif
   GList *tmp;
   GtkWidget *frame;
   int max_label_width; /* the actual max width of the labels we create */
@@ -335,13 +337,19 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   gtk_container_add (GTK_CONTAINER (frame),
                      vbox);
 
+#if GTK_CHECK_VERSION (3, 14, 0)
+  gtk_box_pack_start (GTK_BOX (vbox), grid, TRUE, TRUE, 0);
+#else
   align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 
   gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
+#endif
 
 #if GTK_CHECK_VERSION (3, 0, 0)
+#if !GTK_CHECK_VERSION (3, 14, 0)
   gtk_container_add (GTK_CONTAINER (align),
                      grid);
+#endif
 #else
   gtk_container_add (GTK_CONTAINER (align),
                      table);
@@ -356,7 +364,14 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   obj = gtk_widget_get_accessible (popup->label);
   atk_object_set_role (obj, ATK_ROLE_STATUSBAR);
 
+#if GTK_CHECK_VERSION (3, 14, 0)
+  gtk_widget_set_margin_start (popup->label, 3);
+  gtk_widget_set_margin_end (popup->label, 3);
+  gtk_widget_set_margin_top (popup->label, 3);
+  gtk_widget_set_margin_bottom (popup->label, 3);
+#else
   gtk_misc_set_padding (GTK_MISC (popup->label), 3, 3);
+#endif
 
   gtk_box_pack_end (GTK_BOX (vbox), popup->label, FALSE, FALSE, 0);
 
@@ -382,7 +397,11 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
           if (te->blank)
             {
               /* just stick a widget here to avoid special cases */
+#if GTK_CHECK_VERSION (3, 14, 0)
+              image = gtk_label_new ("");
+#else
               image = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
+#endif
             }
           else if (outline)
             {
@@ -398,7 +417,12 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
               gtk_misc_set_padding (GTK_MISC (image),
                                     INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT + 1,
                                     INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT + 1);
+#if GTK_CHECK_VERSION (3, 0, 0)
+              gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+              gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
+#else
               gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.5);
+#endif
             }
           else
             {
